"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

function _chalk() {
  var data = _interopRequireDefault(require("chalk"));

  _chalk = function _chalk() {
    return data;
  };

  return data;
}

function _commander() {
  var data = _interopRequireDefault(require("commander"));

  _commander = function _commander() {
    return data;
  };

  return data;
}

function _package() {
  var data = _interopRequireDefault(require("../../package.json"));

  _package = function _package() {
    return data;
  };

  return data;
}

function _loader() {
  var data = _interopRequireDefault(require("./loader"));

  _loader = function _loader() {
    return data;
  };

  return data;
}

function _commandList() {
  var data = _interopRequireDefault(require("./commands/command-list"));

  _commandList = function _commandList() {
    return data;
  };

  return data;
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function bitError(message) {
  return _chalk().default.red(`${message.split('\n') // eslint-disable-line
  .map(function (m) {
    return `bitjs ERR! ${m}`;
  }).join('\n')}\n`);
}

function errorHandler(e) {
  _loader().default.off();

  process.stderr.write(bitError(e.message));

  if (e.code) {
    process.stderr.write(bitError(`\ncode: ${e.code}\n`));
  }

  process.stderr.write(bitError(e.stack));
  process.exit(1);
}

function logAndExit(str) {
  _loader().default.off();

  if (str) {
    console.log(str); // eslint-disable-line
  }

  process.exit(0);
}

function start() {
  _commander().default.version(_package().default.version).description('bit driver for javascript');

  _commandList().default.forEach(function (c) {
    var currentCommand = _commander().default.command(c.name).description(c.description);

    if (c.options && Array.isArray(c.options)) {
      c.options.forEach(function (option) {
        return currentCommand.option(`${option.alias} ${option.name}`, option.description);
      });
    }

    currentCommand.action(function (args, options) {
      if (c.loader === true) _loader().default.on();

      _loader().default.start(c.loaderText || `performing ${c.name} command`);

      c.action(args, options).then(c.report).then(logAndExit).catch(c.handleError || errorHandler);
    });
  });

  _commander().default.parse(process.argv);
}

var _default = start;
exports.default = _default;
//# sourceMappingURL=command-registrar.js.map