"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.resolveNodePackage = resolveNodePackage;
exports.resolveModulePath = resolveModulePath;
exports.getDependencyTree = getDependencyTree;

function _regenerator() {
  var data = _interopRequireDefault(require("@babel/runtime/regenerator"));

  _regenerator = function _regenerator() {
    return data;
  };

  return data;
}

function _fs() {
  var data = _interopRequireDefault(require("fs"));

  _fs = function _fs() {
    return data;
  };

  return data;
}

function _path() {
  var data = _interopRequireDefault(require("path"));

  _path = function _path() {
    return data;
  };

  return data;
}

function _ramda() {
  var data = _interopRequireDefault(require("ramda"));

  _ramda = function _ramda() {
    return data;
  };

  return data;
}

function _lodash() {
  var data = _interopRequireDefault(require("lodash.partition"));

  _lodash = function _lodash() {
    return data;
  };

  return data;
}

function _lodash2() {
  var data = _interopRequireDefault(require("lodash.set"));

  _lodash2 = function _lodash2() {
    return data;
  };

  return data;
}

function _generateTreeMadge() {
  var data = _interopRequireWildcard(require("./generate-tree-madge"));

  _generateTreeMadge = function _generateTreeMadge() {
    return data;
  };

  return data;
}

function _packageJson() {
  var data = _interopRequireDefault(require("../package-json/package-json"));

  _packageJson = function _packageJson() {
    return data;
  };

  return data;
}

function _constants() {
  var data = require("../constants");

  _constants = function _constants() {
    return data;
  };

  return data;
}

function _pathMap() {
  var data = require("./path-map");

  _pathMap = function _pathMap() {
    return data;
  };

  return data;
}

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

/**
 * Group dependencies by types (files, bits, packages)
 * @param {any} dependencies list of dependencies paths to group
 * @returns {Function} function which group the dependencies
 */
var byType = function byType(list, bindingPrefix) {
  var grouped = _ramda().default.groupBy(function (item) {
    if (item.includes(`node_modules/${bindingPrefix}`) || item.includes(`node_modules/${_constants().DEFAULT_BINDINGS_PREFIX}`)) {
      return 'bits';
    }

    return item.includes('node_modules') ? 'packages' : 'files';
  });

  return grouped(list);
};
/**
 * Get a path to node package and return the name and version
 *
 * @param {any} packageFullPath full path to the package
 * @returns {Object} name and version of the package
 */


function resolveNodePackage(cwd, packageFullPath) {
  var NODE_MODULES = 'node_modules';
  var result = {}; // Start by searching in the component dir and up from there
  // If not found search in package dir itself.
  // We are doing this, because the package.json insisde the package dir contain exact version
  // And the component/consumer package.json might contain semver like ^ or ~
  // We want to have this semver as dependency and not the exact version, otherwise it will be considered as modified all the time

  var packageJsonInfo = _packageJson().default.findPackage(cwd);

  if (packageJsonInfo) {
    // The +1 is for the / after the node_modules, we didn't enter it into the NODE_MODULES const because it makes problems on windows
    var packageRelativePath = packageFullPath.substring(packageFullPath.lastIndexOf(NODE_MODULES) + NODE_MODULES.length + 1, packageFullPath.length);
    var packageName = resolvePackageNameByPath(packageRelativePath);
    var packageNameNormalized = packageName.replace('\\', '/');

    var packageVersion = _ramda().default.path(['dependencies', packageNameNormalized], packageJsonInfo) || _ramda().default.path(['devDependencies', packageNameNormalized], packageJsonInfo) || _ramda().default.path(['peerDependencies', packageNameNormalized], packageJsonInfo);

    if (packageVersion) {
      result[packageNameNormalized] = packageVersion;
      return result;
    }
  } // Get the package relative path to the node_modules dir


  var packageDir = resolvePackageDirFromFilePath(packageFullPath); // don't propagate here since loading a package.json of another folder and taking the version from it will result wrong version
  // This for example happen in the following case:
  // if you have 2 authored component which one dependet on the other
  // we will look for the package.json on the dependency but won't find it
  // if we propagate we will take the version from the root's package json which has nothing with the component version

  var packageInfo = _packageJson().default.loadSync(packageDir, false); // when running 'bitjs get-dependencies' command, packageInfo is sometimes empty
  // or when using custom-module-resolution it may be empty or the name/version are empty


  if (!packageInfo || !packageInfo.name || !packageInfo.version) return null;
  result[packageInfo.name] = packageInfo.version;
  return result;
}
/**
 * given the full path of a package file, returns the root dir of the package, so then we could
 * find the package.json in that directory.
 *
 * example of a normal package:
 * absolutePackageFilePath: /user/workspace/node_modules/lodash.isboolean/index.js
 * returns: /user/workspace/node_modules/lodash.isboolean
 *
 * example of a scoped package:
 * absolutePackageFilePath: /user/workspace/node_modules/@babel/core/lib/index.js
 * returns: /user/workspace/node_modules/@babel/core
 */


function resolvePackageDirFromFilePath(absolutePackageFilePath) {
  var NODE_MODULES = 'node_modules';
  var indexOfLastNodeModules = absolutePackageFilePath.lastIndexOf(NODE_MODULES) + NODE_MODULES.length + 1;
  var pathInsideNodeModules = absolutePackageFilePath.substring(indexOfLastNodeModules);
  var packageName = resolvePackageNameByPath(pathInsideNodeModules);
  var pathUntilNodeModules = absolutePackageFilePath.substring(0, indexOfLastNodeModules);
  return pathUntilNodeModules + packageName;
}
/**
 * Gets a list of dependencies and group them by types (files, bits, packages)
 * It's also transform the node package dependencies from array of paths to object in this format:
 * {dependencyName: version} (like in package.json)
 *
 * @param {any} list of dependencies paths
 * @param {any} cwd root of working directory (used for node packages version calculation)
 * @returns {Object} object with the dependencies groups
 */


function groupDependencyList(list, cwd, bindingPrefix) {
  var groups = byType(list, bindingPrefix);

  if (groups.packages) {
    var packages = {};
    var unidentifiedPackages = [];
    groups.packages.forEach(function (packagePath) {
      var packageWithVersion = resolveNodePackage(cwd, _path().default.join(cwd, packagePath));
      if (packageWithVersion) Object.assign(packages, packageWithVersion);else unidentifiedPackages.push(packagePath);
    });
    groups.packages = packages;

    if (!_ramda().default.isEmpty(unidentifiedPackages)) {
      groups.unidentifiedPackages = unidentifiedPackages;
    }
  }

  return groups;
}
/**
 * Run over each entry in the tree and transform the dependencies from list of paths
 * to object with dependencies types
 *
 * @param {any} tree
 * @param {any} cwd the working directory path
 * @returns new tree with grouped dependencies
 */


function groupDependencyTree(tree, cwd, bindingPrefix) {
  var result = {};
  Object.keys(tree).forEach(function (key) {
    result[key] = groupDependencyList(tree[key], cwd, bindingPrefix);
  });
  return result;
}
/**
 * Get an import statement path to node package and return the package name
 *
 * @param {string} packagePath import statement path
 * @returns {string} name of the package
 */


function resolvePackageNameByPath(packagePath) {
  var packagePathArr = packagePath.split(_path().default.sep); // TODO: make sure this is working on windows
  // Regular package without path. example - import _ from 'lodash'

  if (packagePathArr.length === 1) return packagePath; // Scoped package. example - import getSymbolIterator from '@angular/core/src/util.d.ts';

  if (packagePathArr[0].startsWith('@')) return _path().default.join(packagePathArr[0], packagePathArr[1]); // Regular package with internal path. example import something from 'mypackage/src/util/isString'

  return packagePathArr[0];
}
/**
 * Recursively search for node module inside node_modules dir
 * This function propegate up until it get's to the root provided then stops
 *
 * @param {string} nmPath - package name
 * @param {string} workingDir - dir to start searching of
 * @param {string} root - path to dir to stop the search
 * @returns The resolved path for the package directory
 */


function resolveModulePath(nmPath, workingDir, root) {
  var pathToCheck = _path().default.resolve(workingDir, 'node_modules', nmPath);

  if (_fs().default.existsSync(pathToCheck)) {
    return pathToCheck;
  }

  if (workingDir === root) {
    return null;
  }

  var parentWorkingDir = _path().default.dirname(workingDir);

  if (parentWorkingDir === workingDir) return null;
  return resolveModulePath(nmPath, parentWorkingDir, root);
}
/**
 * Resolve package dependencies from package.json according to package names
 *
 * @param {Object} packageJson
 * @param {string []} packagesNames
 * @returns new object with found and missing
 */


function findPackagesInPackageJson(packageJson, packagesNames) {
  var dependencies = packageJson.dependencies,
      devDependencies = packageJson.devDependencies,
      peerDependencies = packageJson.peerDependencies;
  var foundPackages = {};
  var mergedDependencies = Object.assign({}, dependencies, devDependencies, peerDependencies);

  if (packagesNames && packagesNames.length && !_ramda().default.isNil(mergedDependencies)) {
    var _partition = (0, _lodash().default)(packagesNames, function (item) {
      return item in mergedDependencies;
    }),
        _partition2 = _slicedToArray(_partition, 2),
        foundPackagesPartition = _partition2[0],
        missingPackages = _partition2[1];

    foundPackagesPartition.forEach(function (pack) {
      return foundPackages[pack] = mergedDependencies[pack];
    });
    return {
      foundPackages,
      missingPackages
    };
  }

  return {
    foundPackages: {},
    missingPackages: packagesNames
  };
}
/**
 * Run over each entry in the missing array and transform the missing from list of paths
 * to object with missing types
 *
 * @param {Array} missing
 * @param {string} cwd
 * @param {string} consumerPath
 * @param {string} bindingPrefix
 * @returns new object with grouped missing
 */


function groupMissing(missing, cwd, consumerPath, bindingPrefix) {
  // temporarily disable this functionality since it cause few bugs: explanation below (on using the packageJson)
  // const packageJson = PackageJson.findPackage(cwd);

  /**
   * Group missing dependencies by types (files, bits, packages)
   * @param {Array} missing list of missing paths to group
   * @returns {Function} function which group the dependencies
   */
  var byPathType = _ramda().default.groupBy(function (item) {
    if (item.startsWith(`${bindingPrefix}/`) || item.startsWith(`${_constants().DEFAULT_BINDINGS_PREFIX}/`)) return 'bits';
    return item.startsWith('.') ? 'files' : 'packages';
  });

  var groups = Object.keys(missing).map(function (key) {
    return Object.assign({
      originFile: (0, _generateTreeMadge().processPath)(key, {}, cwd)
    }, byPathType(missing[key], bindingPrefix));
  });
  groups.forEach(function (group) {
    if (group.packages) group.packages = group.packages.map(resolvePackageNameByPath);
  }); // This is a hack to solve problems that madge has with packages for type script files
  // It see them as missing even if they are exists

  var foundPackages = {};

  var packageJson = _packageJson().default.findPackage(cwd);

  groups.forEach(function (group) {
    var missingPackages = [];

    if (group.packages) {
      group.packages.forEach(function (packageName) {
        // Don't add the same package twice
        if (_ramda().default.contains(packageName, missingPackages)) return;
        var resolvedPath = resolveModulePath(packageName, cwd, consumerPath);

        if (!resolvedPath) {
          return missingPackages.push(packageName);
        }

        var packageWithVersion = resolveNodePackage(cwd, resolvedPath);
        return packageWithVersion ? Object.assign(foundPackages, packageWithVersion) : missingPackages.push(packageWithVersion);
      });
    }

    if (packageJson) {
      var result = findPackagesInPackageJson(packageJson, missingPackages);
      groups.packages = result.missingPackages;
      Object.assign(foundPackages, result.foundPackages);
    }
  }); // temporarily disable this functionality since it cause this bugs:
  // https://github.com/teambit/bit/issues/635
  // https://github.com/teambit/bit/issues/690

  return {
    missingGroups: groups,
    foundPackages
  };
}
/**
 * add extra data such as custom-resolve and link-files from pathMap
 */


function updateTreeWithPathMap(tree, pathMapAbsolute, baseDir) {
  if (!pathMapAbsolute.length) return;
  var pathMapRelative = (0, _pathMap().convertPathMapToRelativePaths)(pathMapAbsolute, baseDir);
  var pathMap = (0, _pathMap().getPathMapWithLinkFilesData)(pathMapRelative);
  Object.keys(tree).forEach(function (filePath) {
    var treeFiles = tree[filePath].files;
    if (!treeFiles || !treeFiles.length) return; // file has no dependency

    var mainFilePathMap = pathMap.find(function (file) {
      return file.file === filePath;
    });
    if (!mainFilePathMap) throw new Error(`updateTreeWithPathMap: PathMap is missing for ${filePath}`); // a file might have a cycle dependency with itself, remove it from the dependencies.

    var files = treeFiles.filter(function (dependency) {
      return dependency !== filePath;
    }).map(function (dependency) {
      var dependencyPathMap = mainFilePathMap.dependencies.find(function (file) {
        return file.resolvedDep === dependency;
      });
      if (!dependencyPathMap) throw new Error(`updateTreeWithPathMap: dependencyPathMap is missing for ${dependency}`);
      var fileObject = {
        file: dependency,
        importSource: dependencyPathMap.importSource,
        isCustomResolveUsed: dependencyPathMap.isCustomResolveUsed
      };

      if (dependencyPathMap.linkFile) {
        fileObject.isLink = true;
        fileObject.linkDependencies = dependencyPathMap.realDependencies;
        return fileObject;
      }

      if (dependencyPathMap.importSpecifiers && dependencyPathMap.importSpecifiers.length) {
        var depImportSpecifiers = dependencyPathMap.importSpecifiers.map(function (importSpecifier) {
          return {
            mainFile: importSpecifier
          };
        });
        fileObject.importSpecifiers = depImportSpecifiers;
      }

      return fileObject;
    });
    tree[filePath].files = files; // eslint-disable-line no-param-reassign
  });
}
/**
 * config aliases are passed later on to webpack-enhancer and it expects them to have the full path
 */


function getResolveConfigAbsolute(consumerPath, resolveConfig) {
  if (!resolveConfig) return resolveConfig;

  var resolveConfigAbsolute = _ramda().default.clone(resolveConfig);

  if (resolveConfig.modulesDirectories) {
    resolveConfigAbsolute.modulesDirectories = resolveConfig.modulesDirectories.map(function (moduleDirectory) {
      return _path().default.isAbsolute(moduleDirectory) ? moduleDirectory : _path().default.join(consumerPath, moduleDirectory);
    });
  }

  if (resolveConfigAbsolute.aliases) {
    Object.keys(resolveConfigAbsolute.aliases).forEach(function (alias) {
      if (!_path().default.isAbsolute(resolveConfigAbsolute.aliases[alias])) {
        resolveConfigAbsolute.aliases[alias] = _path().default.join(consumerPath, resolveConfigAbsolute.aliases[alias]);
      }
    });
  }

  return resolveConfigAbsolute;
}

function mergeManuallyFoundPackagesToTree(foundPackages, groups, tree) {
  if (_ramda().default.isEmpty(foundPackages)) return; // Merge manually found packages (by groupMissing()) with the packages found by Madge (generate-tree-madge)

  Object.keys(foundPackages).forEach(function (pkg) {
    // locate package in groups(contains missing)
    groups.forEach(function (fileDep) {
      if (fileDep.packages && fileDep.packages.includes(pkg)) {
        fileDep.packages = fileDep.packages.filter(function (packageName) {
          return packageName !== pkg;
        });
        (0, _lodash2().default)(tree[fileDep.originFile], ['packages', pkg], foundPackages[pkg]);
      }
    });
  });
}

function mergeMissingToTree(missingGroups, tree) {
  if (_ramda().default.isEmpty(missingGroups)) return;
  missingGroups.forEach(function (missing) {
    var missingCloned = _ramda().default.clone(missing);

    delete missingCloned.originFile;
    if (tree[missing.originFile]) tree[missing.originFile].missing = missingCloned;else tree[missing.originFile] = {
      missing: missingCloned
    };
  });
}

function mergeErrorsToTree(baseDir, errors, tree) {
  if (_ramda().default.isEmpty(errors)) return;
  Object.keys(errors).forEach(function (file) {
    if (tree[file]) tree[file].error = errors[file];else tree[file] = {
      error: errors[file]
    };
  });
}
/**
 * Function for fetching dependency tree of file or dir
 * @param baseDir working directory
 * @param consumerPath
 * @param filePaths path of the file to calculate the dependencies
 * @param bindingPrefix
 * @return {Promise<{missing, tree}>}
 */


function getDependencyTree(_x) {
  return _getDependencyTree.apply(this, arguments);
}

function _getDependencyTree() {
  _getDependencyTree = _asyncToGenerator(
  /*#__PURE__*/
  _regenerator().default.mark(function _callee(_ref) {
    var baseDir, consumerPath, filePaths, bindingPrefix, resolveModulesConfig, _ref$visited, visited, resolveConfigAbsolute, config, _generateTree, madgeTree, skipped, pathMap, errors, tree, _groupMissing, missingGroups, foundPackages;

    return _regenerator().default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            baseDir = _ref.baseDir, consumerPath = _ref.consumerPath, filePaths = _ref.filePaths, bindingPrefix = _ref.bindingPrefix, resolveModulesConfig = _ref.resolveModulesConfig, _ref$visited = _ref.visited, visited = _ref$visited === void 0 ? {} : _ref$visited;
            resolveConfigAbsolute = getResolveConfigAbsolute(consumerPath, resolveModulesConfig);
            config = {
              baseDir,
              includeNpm: true,
              requireConfig: null,
              webpackConfig: null,
              visited,
              nonExistent: [],
              resolveConfig: resolveConfigAbsolute
            };
            _generateTree = (0, _generateTreeMadge().default)(filePaths, config), madgeTree = _generateTree.madgeTree, skipped = _generateTree.skipped, pathMap = _generateTree.pathMap, errors = _generateTree.errors;
            tree = groupDependencyTree(madgeTree, baseDir, bindingPrefix);
            _groupMissing = groupMissing(skipped, baseDir, consumerPath, bindingPrefix), missingGroups = _groupMissing.missingGroups, foundPackages = _groupMissing.foundPackages;
            if (foundPackages) mergeManuallyFoundPackagesToTree(foundPackages, missingGroups, tree);
            if (errors) mergeErrorsToTree(baseDir, errors, tree);
            if (missingGroups) mergeMissingToTree(missingGroups, tree);
            if (pathMap) updateTreeWithPathMap(tree, pathMap, baseDir);
            return _context.abrupt("return", {
              tree
            });

          case 11:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));
  return _getDependencyTree.apply(this, arguments);
}
//# sourceMappingURL=build-tree.js.map