"use strict";

function _regenerator() {
  var data = _interopRequireDefault(require("@babel/runtime/regenerator"));

  _regenerator = function _regenerator() {
    return data;
  };

  return data;
}

function _path() {
  var data = _interopRequireDefault(require("path"));

  _path = function _path() {
    return data;
  };

  return data;
}

function _chai() {
  var data = require("chai");

  _chai = function _chai() {
    return data;
  };

  return data;
}

function buildTree() {
  var data = _interopRequireWildcard(require("./build-tree"));

  buildTree = function buildTree() {
    return data;
  };

  return data;
}

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

var fixtures = `${__dirname}/../../fixtures`;

var precinctFixtures = _path().default.join(fixtures, 'precinct');

var buildTreeFixtures = _path().default.join(fixtures, 'build-tree');

describe('buildTree', function () {
  describe('getDependencyTree', function () {
    var dependencyTreeParams = {
      baseDir: '.',
      consumerPath: __dirname,
      filePaths: [],
      bindingPrefix: '@bit',
      resolveModulesConfig: undefined
    };
    it('when no files are passed should return an empty tree',
    /*#__PURE__*/
    _asyncToGenerator(
    /*#__PURE__*/
    _regenerator().default.mark(function _callee() {
      var results;
      return _regenerator().default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              _context.next = 2;
              return buildTree().getDependencyTree(dependencyTreeParams);

            case 2:
              results = _context.sent;
              (0, _chai().expect)(results).to.deep.equal({
                tree: {}
              });

            case 4:
            case "end":
              return _context.stop();
          }
        }
      }, _callee);
    })));
    it('when unsupported files are passed should return them with no dependencies',
    /*#__PURE__*/
    _asyncToGenerator(
    /*#__PURE__*/
    _regenerator().default.mark(function _callee2() {
      var results;
      return _regenerator().default.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              dependencyTreeParams.filePaths = [`${fixtures}/unsupported-file.pdf`];
              _context2.next = 3;
              return buildTree().getDependencyTree(dependencyTreeParams);

            case 3:
              results = _context2.sent;
              (0, _chai().expect)(results.tree).to.deep.equal({
                'fixtures/unsupported-file.pdf': {}
              });

            case 5:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2);
    })));
    it('when supported and unsupported files are passed should return them all',
    /*#__PURE__*/
    _asyncToGenerator(
    /*#__PURE__*/
    _regenerator().default.mark(function _callee3() {
      var results;
      return _regenerator().default.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              dependencyTreeParams.filePaths = [`${fixtures}/unsupported-file.pdf`, `${precinctFixtures}/es6.js`];
              _context3.next = 3;
              return buildTree().getDependencyTree(dependencyTreeParams);

            case 3:
              results = _context3.sent;
              (0, _chai().expect)(results.tree).to.have.property('fixtures/unsupported-file.pdf');
              (0, _chai().expect)(results.tree).to.have.property('fixtures/precinct/es6.js');

            case 6:
            case "end":
              return _context3.stop();
          }
        }
      }, _callee3);
    })));
    it('when a js file has parsing error it should add the file to the tree with the error instance',
    /*#__PURE__*/
    _asyncToGenerator(
    /*#__PURE__*/
    _regenerator().default.mark(function _callee4() {
      var results, unParsedFile;
      return _regenerator().default.wrap(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              dependencyTreeParams.filePaths = [`${precinctFixtures}/unparseable.js`];
              _context4.next = 3;
              return buildTree().getDependencyTree(dependencyTreeParams);

            case 3:
              results = _context4.sent;
              unParsedFile = 'fixtures/precinct/unparseable.js';
              (0, _chai().expect)(results.tree).to.have.property(unParsedFile);
              (0, _chai().expect)(results.tree[unParsedFile]).to.have.property('error');
              (0, _chai().expect)(results.tree[unParsedFile].error).to.be.instanceof(Error);

            case 8:
            case "end":
              return _context4.stop();
          }
        }
      }, _callee4);
    })));
    it('when a js file has parsing error and it retrieved from the cache it should add the file to the tree with the error instance',
    /*#__PURE__*/
    _asyncToGenerator(
    /*#__PURE__*/
    _regenerator().default.mark(function _callee5() {
      var results, unParsedFile, resultsCached;
      return _regenerator().default.wrap(function _callee5$(_context5) {
        while (1) {
          switch (_context5.prev = _context5.next) {
            case 0:
              dependencyTreeParams.filePaths = [`${precinctFixtures}/unparseable.js`];
              dependencyTreeParams.visited = {};
              _context5.next = 4;
              return buildTree().getDependencyTree(dependencyTreeParams);

            case 4:
              results = _context5.sent;
              unParsedFile = 'fixtures/precinct/unparseable.js';
              (0, _chai().expect)(results.tree).to.have.property(unParsedFile);
              (0, _chai().expect)(results.tree[unParsedFile]).to.have.property('error');
              (0, _chai().expect)(results.tree[unParsedFile].error).to.be.instanceof(Error); // second time, this time it fetches from the cache (visited object)

              _context5.next = 11;
              return buildTree().getDependencyTree(dependencyTreeParams);

            case 11:
              resultsCached = _context5.sent;
              (0, _chai().expect)(resultsCached.tree).to.have.property(unParsedFile);
              (0, _chai().expect)(resultsCached.tree[unParsedFile]).to.have.property('error');
              (0, _chai().expect)(resultsCached.tree[unParsedFile].error).to.be.instanceof(Error);

            case 15:
            case "end":
              return _context5.stop();
          }
        }
      }, _callee5);
    })));
    it.skip('when a css file has parsing error it should add the file to the tree with the error instance',
    /*#__PURE__*/
    _asyncToGenerator(
    /*#__PURE__*/
    _regenerator().default.mark(function _callee6() {
      var results, unParsedFile;
      return _regenerator().default.wrap(function _callee6$(_context6) {
        while (1) {
          switch (_context6.prev = _context6.next) {
            case 0:
              dependencyTreeParams.filePaths = [`${buildTreeFixtures}/unparsed.css`];
              _context6.next = 3;
              return buildTree().getDependencyTree(dependencyTreeParams);

            case 3:
              results = _context6.sent;
              unParsedFile = 'fixtures/build-tree/unparsed.css';
              (0, _chai().expect)(results.tree).to.have.property(unParsedFile);
              (0, _chai().expect)(results.tree[unParsedFile]).to.have.property('error');
              (0, _chai().expect)(results.tree[unParsedFile].error).to.be.instanceof(Error);

            case 8:
            case "end":
              return _context6.stop();
          }
        }
      }, _callee6);
    })));
    describe('when a dependency of dependency has parsing error', function () {
      var results;
      before(
      /*#__PURE__*/
      _asyncToGenerator(
      /*#__PURE__*/
      _regenerator().default.mark(function _callee7() {
        return _regenerator().default.wrap(function _callee7$(_context7) {
          while (1) {
            switch (_context7.prev = _context7.next) {
              case 0:
                dependencyTreeParams.filePaths = [`${buildTreeFixtures}/a.js`, `${buildTreeFixtures}/b.js`];
                _context7.next = 3;
                return buildTree().getDependencyTree(dependencyTreeParams);

              case 3:
                results = _context7.sent;

              case 4:
              case "end":
                return _context7.stop();
            }
          }
        }, _callee7);
      })));
      it('should add all the files to the tree',
      /*#__PURE__*/
      _asyncToGenerator(
      /*#__PURE__*/
      _regenerator().default.mark(function _callee8() {
        return _regenerator().default.wrap(function _callee8$(_context8) {
          while (1) {
            switch (_context8.prev = _context8.next) {
              case 0:
                (0, _chai().expect)(results.tree).to.have.property('fixtures/build-tree/a.js');
                (0, _chai().expect)(results.tree).to.have.property('fixtures/build-tree/b.js');
                (0, _chai().expect)(results.tree).to.have.property('fixtures/build-tree/unparsed.js');

              case 3:
              case "end":
                return _context8.stop();
            }
          }
        }, _callee8);
      })));
      it('should not add the error to the files without parsing error', function () {
        (0, _chai().expect)(results.tree['fixtures/build-tree/a.js']).to.not.have.property('error');
        (0, _chai().expect)(results.tree['fixtures/build-tree/b.js']).to.not.have.property('error');
      });
      it('should add the parsing error to the un-parsed file', function () {
        (0, _chai().expect)(results.tree['fixtures/build-tree/unparsed.js']).to.have.property('error');
        (0, _chai().expect)(results.tree['fixtures/build-tree/unparsed.js'].error).to.be.instanceof(Error);
      });
    });
    describe('missing dependencies', function () {
      var results;
      var missingDepsFile = 'fixtures/missing-deps.js';
      before(
      /*#__PURE__*/
      _asyncToGenerator(
      /*#__PURE__*/
      _regenerator().default.mark(function _callee9() {
        return _regenerator().default.wrap(function _callee9$(_context9) {
          while (1) {
            switch (_context9.prev = _context9.next) {
              case 0:
                dependencyTreeParams.filePaths = [`${fixtures}/missing-deps.js`];
                _context9.next = 3;
                return buildTree().getDependencyTree(dependencyTreeParams);

              case 3:
                results = _context9.sent;
                (0, _chai().expect)(results.tree).to.have.property(missingDepsFile);
                (0, _chai().expect)(results.tree[missingDepsFile]).to.have.property('missing');

              case 6:
              case "end":
                return _context9.stop();
            }
          }
        }, _callee9);
      })));
      it('it should add the missing dependency to the missing section in the tree',
      /*#__PURE__*/
      _asyncToGenerator(
      /*#__PURE__*/
      _regenerator().default.mark(function _callee10() {
        return _regenerator().default.wrap(function _callee10$(_context10) {
          while (1) {
            switch (_context10.prev = _context10.next) {
              case 0:
                (0, _chai().expect)(results.tree[missingDepsFile].missing).to.have.property('files');
                (0, _chai().expect)(results.tree[missingDepsFile].missing.files[0]).to.equal('../non-exist-dep');

              case 2:
              case "end":
                return _context10.stop();
            }
          }
        }, _callee10);
      })));
      it('it should add the missing package to the missing section in the tree',
      /*#__PURE__*/
      _asyncToGenerator(
      /*#__PURE__*/
      _regenerator().default.mark(function _callee11() {
        return _regenerator().default.wrap(function _callee11$(_context11) {
          while (1) {
            switch (_context11.prev = _context11.next) {
              case 0:
                (0, _chai().expect)(results.tree[missingDepsFile].missing).to.have.property('packages');
                (0, _chai().expect)(results.tree[missingDepsFile].missing.packages[0]).to.equal('non-exist-package');

              case 2:
              case "end":
                return _context11.stop();
            }
          }
        }, _callee11);
      })));
    });
    describe('tree shaking with cycle', function () {
      describe('when a file imports from itself', function () {
        var results;
        before(
        /*#__PURE__*/
        _asyncToGenerator(
        /*#__PURE__*/
        _regenerator().default.mark(function _callee12() {
          return _regenerator().default.wrap(function _callee12$(_context12) {
            while (1) {
              switch (_context12.prev = _context12.next) {
                case 0:
                  dependencyTreeParams.filePaths = [`${buildTreeFixtures}/tree-shaking-cycle/self-cycle.js`];
                  _context12.next = 3;
                  return buildTree().getDependencyTree(dependencyTreeParams);

                case 3:
                  results = _context12.sent;

                case 4:
                case "end":
                  return _context12.stop();
              }
            }
          }, _callee12);
        })));
        it('should not throw an error and should remove itself from the dependencies files', function () {
          var file = 'fixtures/build-tree/tree-shaking-cycle/self-cycle.js';
          (0, _chai().expect)(results.tree[file].files).to.be.an('array').and.empty;
        });
      });
      describe('cycle with multiple files', function () {
        var results;
        before(
        /*#__PURE__*/
        _asyncToGenerator(
        /*#__PURE__*/
        _regenerator().default.mark(function _callee13() {
          return _regenerator().default.wrap(function _callee13$(_context13) {
            while (1) {
              switch (_context13.prev = _context13.next) {
                case 0:
                  dependencyTreeParams.filePaths = [`${buildTreeFixtures}/tree-shaking-cycle/foo.js`];
                  _context13.next = 3;
                  return buildTree().getDependencyTree(dependencyTreeParams);

                case 3:
                  results = _context13.sent;

                case 4:
                case "end":
                  return _context13.stop();
              }
            }
          }, _callee13);
        })));
        it('should not recognize the cycle dependencies as link files', function () {
          var file = 'fixtures/build-tree/tree-shaking-cycle/foo.js';
          (0, _chai().expect)(results.tree[file].files).to.be.an('array').and.have.lengthOf(1);
          var indexDep = results.tree[file].files[0];
          (0, _chai().expect)(indexDep.file).to.equal('fixtures/build-tree/tree-shaking-cycle/index.js');
          (0, _chai().expect)(indexDep).to.not.have.property('isLink');
          (0, _chai().expect)(indexDep).to.not.have.property('linkDependencies');
        });
      });
    });
  });
});
//# sourceMappingURL=build-tree.spec.js.map