"use strict";

function _assert() {
  var data = _interopRequireDefault(require("assert"));

  _assert = function _assert() {
    return data;
  };

  return data;
}

function _chai() {
  var data = require("chai");

  _chai = function _chai() {
    return data;
  };

  return data;
}

function _sinon() {
  var data = _interopRequireDefault(require("sinon"));

  _sinon = function _sinon() {
    return data;
  };

  return data;
}

function _mockFs() {
  var data = _interopRequireDefault(require("mock-fs"));

  _mockFs = function _mockFs() {
    return data;
  };

  return data;
}

function _path() {
  var data = _interopRequireDefault(require("path"));

  _path = function _path() {
    return data;
  };

  return data;
}

function _precinct() {
  var data = _interopRequireDefault(require("../precinct"));

  _precinct = function _precinct() {
    return data;
  };

  return data;
}

function _rewire() {
  var data = _interopRequireDefault(require("rewire"));

  _rewire = function _rewire() {
    return data;
  };

  return data;
}

function _Config() {
  var data = _interopRequireDefault(require("./Config"));

  _Config = function _Config() {
    return data;
  };

  return data;
}

function _resolve() {
  var data = _interopRequireDefault(require("resolve"));

  _resolve = function _resolve() {
    return data;
  };

  return data;
}

function _typescript() {
  var data = _interopRequireDefault(require("typescript"));

  _typescript = function _typescript() {
    return data;
  };

  return data;
}

function _moduleDefinition() {
  var data = _interopRequireDefault(require("module-definition"));

  _moduleDefinition = function _moduleDefinition() {
    return data;
  };

  return data;
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Bootstrap lazy requires
var dependencyTree = (0, _rewire().default)('./');

var fixtures = _path().default.resolve(`${__dirname}/../../../fixtures/dependency-tree`);

describe('dependencyTree', function () {
  var _this = this;

  this.timeout(8000);

  function testTreesForFormat(format) {
    var ext = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '.js';
    it('returns an object form of the dependency tree for a file', function () {
      var root = `${fixtures}/${format}`;

      var filename = _path().default.normalize(`${root}/a${ext}`);

      var tree = dependencyTree({
        filename,
        root
      });
      (0, _assert().default)(tree instanceof Object);
      var aSubTree = tree[filename];

      _assert().default.ok(aSubTree instanceof Object);

      var filesInSubTree = Object.keys(aSubTree);

      _assert().default.equal(filesInSubTree.length, 2);
    });
  }

  function mockStylus() {
    (0, _mockFs().default)({
      [`${fixtures}/stylus`]: {
        'a.styl': `
          @import "b"
          @require "c.styl"
        `,
        'b.styl': '@import "c"',
        'c.styl': ''
      }
    });
  }

  function mockSass() {
    (0, _mockFs().default)({
      [`${fixtures}/sass`]: {
        'a.scss': `
          @import "_b";
          @import "_c.scss";
        `,
        '_b.scss': 'body { color: blue; }',
        '_c.scss': 'body { color: pink; }'
      }
    });
  }

  function mockLess() {
    (0, _mockFs().default)({
      [`${fixtures}/less`]: {
        'a.less': `
          @import "b.css";
          @import "c.less";
        `,
        'b.css': 'body { color: blue; }',
        'c.less': 'body { color: pink; }'
      }
    });
  }

  function mockes6() {
    (0, _mockFs().default)({
      [`${fixtures}/es6`]: {
        'a.js': `
          import b from './b';
          import c from './c';
        `,
        'b.js': 'export default () => {};',
        'c.js': 'export default () => {};',
        'jsx.js': "import c from './c';\n export default <jsx />;",
        'foo.jsx': "import React from 'react';\n import b from './b';\n export default <jsx />;",
        'es7.js': "import c from './c';\n export default async function foo() {};"
      }
    });
  }

  function mockTS() {
    (0, _mockFs().default)({
      [`${fixtures}/ts`]: {
        'a.ts': `
          import b from './b';
          import c from './c';
        `,
        'b.ts': 'export default () => {};',
        'c.ts': 'export default () => {};'
      }
    });
  }

  afterEach(function () {
    _mockFs().default.restore();
  });
  it('returns an empty object for a non-existent filename', function () {
    (0, _mockFs().default)({
      imaginary: {}
    });
    var root = `${__dirname}/imaginary`;
    var filename = `${root}/notafile.js`;
    var tree = dependencyTree({
      filename,
      root
    });
    (0, _assert().default)(tree instanceof Object);
    (0, _assert().default)(!Object.keys(tree).length);
  });
  it('handles nested tree structures', function () {
    (0, _mockFs().default)({
      [`${__dirname}/extended`]: {
        'a.js': `var b = require('./b');
                 var c = require('./c');`,
        'b.js': `var d = require('./d');
                 var e = require('./e');`,
        'c.js': `var f = require('./f');
                 var g = require('./g');`,
        'd.js': '',
        'e.js': '',
        'f.js': '',
        'g.js': ''
      }
    });
    var directory = `${__dirname}/extended`;

    var filename = _path().default.normalize(`${directory}/a.js`);

    var tree = dependencyTree({
      filename,
      directory
    });
    (0, _assert().default)(tree[filename] instanceof Object); // b and c

    var subTree = tree[filename];

    _assert().default.equal(subTree.length, 2);

    var bTree = tree[_path().default.normalize(`${directory}/b.js`)];

    var cTree = tree[_path().default.normalize(`${directory}/c.js`)]; // d and e


    _assert().default.equal(bTree.length, 2); // f ang g


    _assert().default.equal(cTree.length, 2);
  });
  it('does not include files that are not real (#13)', function () {
    (0, _mockFs().default)({
      [`${__dirname}/onlyRealDeps`]: {
        'a.js': 'var notReal = require("./notReal");'
      }
    });
    var directory = `${__dirname}/onlyRealDeps`;

    var filename = _path().default.normalize(`${directory}/a.js`);

    var tree = dependencyTree({
      filename,
      directory
    });
    var subTree = tree[filename];

    _assert().default.ok(!Object.keys(subTree).some(function (dep) {
      return dep.indexOf('notReal') !== -1;
    }));
  });
  it('does not choke on cyclic dependencies', function () {
    (0, _mockFs().default)({
      [`${__dirname}/cyclic`]: {
        'a.js': 'var b = require("./b");',
        'b.js': 'var a = require("./a");'
      }
    });
    var directory = `${__dirname}/cyclic`;

    var filename = _path().default.normalize(`${directory}/a.js`);

    var spy = _sinon().default.spy(dependencyTree, '_getDependencies');

    var tree = dependencyTree({
      filename,
      directory
    });
    (0, _assert().default)(spy.callCount === 2);
    (0, _assert().default)(Object.keys(tree[filename]).length);

    dependencyTree._getDependencies.restore();
  });
  it('excludes Nodejs core modules by default', function () {
    var directory = `${fixtures}/commonjs`;

    var filename = _path().default.normalize(`${directory}/b.js`);

    var tree = dependencyTree({
      filename,
      directory
    });
    (0, _assert().default)(Object.keys(tree[filename]).length === 0);
    (0, _assert().default)(Object.keys(tree)[0].indexOf('b.js') !== -1);
  });
  it('traverses installed 3rd party node modules', function () {
    var directory = `${fixtures}/onlyRealDeps`;

    var filename = _path().default.normalize(`${directory}/a.js`);

    var tree = dependencyTree({
      filename,
      directory
    });
    var subTree = tree[filename];
    (0, _assert().default)(subTree.includes(require.resolve('debug')));
  });
  it('returns a list of absolutely pathed files', function () {
    var directory = `${fixtures}/commonjs`;
    var filename = `${directory}/b.js`;
    var tree = dependencyTree({
      filename,
      directory
    });

    for (var node in tree.nodes) {
      (0, _assert().default)(node.indexOf(process.cwd()) !== -1);
    }
  });
  describe('when given a detective configuration', function () {
    it('passes it through to precinct', function () {
      var spy = _sinon().default.spy(_precinct().default, 'paperwork');

      var directory = _path().default.normalize(`${fixtures}/onlyRealDeps`);

      var filename = _path().default.normalize(`${directory}/a.js`);

      var detectiveConfig = {
        amd: {
          skipLazyLoaded: true
        }
      };
      dependencyTree({
        filename,
        directory,
        detective: detectiveConfig
      });

      _assert().default.ok(spy.calledWith(filename, detectiveConfig));

      spy.restore();
    });
  });
  describe('when given a list to store non existent partials', function () {
    describe('and the file contains no valid partials', function () {
      it('stores the invalid partials', function () {
        (0, _mockFs().default)({
          [`${__dirname}/onlyRealDeps`]: {
            'a.js': 'var notReal = require("./notReal");'
          }
        });

        var directory = _path().default.normalize(`${__dirname}/onlyRealDeps`);

        var filename = _path().default.normalize(`${directory}/a.js`);

        var nonExistent = [];
        var tree = dependencyTree({
          filename,
          directory,
          nonExistent
        });

        _assert().default.equal(Object.keys(nonExistent).length, 1);

        _assert().default.equal(nonExistent[filename][0], './notReal');
      });
    });
    describe('and the file contains all valid partials', function () {
      it('does not store anything', function () {
        (0, _mockFs().default)({
          [`${__dirname}/onlyRealDeps`]: {
            'a.js': 'var b = require("./b");',
            'b.js': 'export default 1;'
          }
        });
        var directory = `${__dirname}/onlyRealDeps`;
        var filename = `${directory}/a.js`;
        var nonExistent = [];
        var tree = dependencyTree({
          filename,
          directory,
          nonExistent
        });

        _assert().default.equal(nonExistent.length, 0);
      });
    });
    describe('and the file contains a mix of invalid and valid partials', function () {
      it('stores the invalid ones', function () {
        (0, _mockFs().default)({
          [`${__dirname}/onlyRealDeps`]: {
            'a.js': 'var b = require("./b");',
            'b.js': 'var c = require("./c"); export default 1;',
            'c.js': 'var crap = require("./notRealMan");'
          }
        });

        var directory = _path().default.normalize(`${__dirname}/onlyRealDeps`);

        var filename = _path().default.normalize(`${directory}/a.js`);

        var nonExistent = [];
        var tree = dependencyTree({
          filename,
          directory,
          nonExistent
        });

        _assert().default.equal(Object.keys(nonExistent).length, 1);

        _assert().default.equal(nonExistent[_path().default.normalize(`${directory}/c.js`)][0], './notRealMan');
      });
    });
    describe('and there is more than one reference to the invalid partial', function () {
      it('should include the non-existent partial per file', function () {
        (0, _mockFs().default)({
          [`${__dirname}/onlyRealDeps`]: {
            'a.js': 'var b = require("./b");\nvar crap = require("./notRealMan");',
            'b.js': 'var c = require("./c"); export default 1;',
            'c.js': 'var crap = require("./notRealMan");'
          }
        });

        var directory = _path().default.normalize(`${__dirname}/onlyRealDeps`);

        var filename = _path().default.normalize(`${directory}/a.js`);

        var nonExistent = [];
        var tree = dependencyTree({
          filename,
          directory,
          nonExistent
        });

        _assert().default.equal(Object.keys(nonExistent).length, 2);

        _assert().default.equal(nonExistent[filename][0], './notRealMan');

        _assert().default.equal(nonExistent[_path().default.normalize(`${directory}/c.js`)][0], './notRealMan');
      });
    });
  });
  describe('throws', function () {
    beforeEach(function () {
      _this._directory = `${fixtures}/commonjs`;
      _this._revert = dependencyTree.__set__('traverse', function () {
        return [];
      });
    });
    afterEach(function () {
      _this._revert();
    });
    it('throws if the filename is missing', function () {
      _assert().default.throws(function () {
        dependencyTree({
          filename: undefined,
          directory: _this._directory
        });
      });
    });
    it('throws if the root is missing', function () {
      _assert().default.throws(function () {
        dependencyTree({
          filename
        });
      });
    });
    it('throws if a supplied filter is not a function', function () {
      _assert().default.throws(function () {
        var directory = `${fixtures}/onlyRealDeps`;
        var filename = `${directory}/a.js`;
        var tree = dependencyTree({
          filename,
          directory,
          filter: 'foobar'
        });
      });
    });
    it('does not throw on the legacy `root` option', function () {
      _assert().default.doesNotThrow(function () {
        var directory = `${fixtures}/onlyRealDeps`;
        var filename = `${directory}/a.js`;
        var tree = dependencyTree({
          filename,
          root: directory
        });
      });
    });
  });
  describe('on file error', function () {
    beforeEach(function () {
      _this._directory = `${fixtures}/commonjs`;
    });
    it('does not throw', function () {
      _assert().default.doesNotThrow(function () {
        dependencyTree({
          filename: 'foo',
          directory: _this._directory
        });
      });
    });
    it('returns no dependencies', function () {
      var tree = dependencyTree({
        filename: 'foo',
        directory: _this._directory
      });
      (0, _assert().default)(!tree.length);
    });
  });
  describe('memoization (#2)', function () {
    beforeEach(function () {
      _this._spy = _sinon().default.spy(dependencyTree, '_getDependencies');
    });
    afterEach(function () {
      dependencyTree._getDependencies.restore();
    });
    it('accepts a cache object for memoization (#2)', function () {
      var filename = _path().default.normalize(`${fixtures}/amd/a.js`);

      var directory = _path().default.normalize(`${fixtures}/amd`);

      var cache = {};
      cache[_path().default.normalize(`${fixtures}/amd/b.js`)] = {
        pathMap: {
          dependencies: [_path().default.normalize(`${fixtures}/amd/b.js`), _path().default.normalize(`${fixtures}/amd/c.js`)]
        }
      };
      var tree = dependencyTree({
        filename,
        directory,
        visited: cache
      });

      _assert().default.equal(Object.keys(tree[filename]).length, 2);

      (0, _assert().default)(_this._spy.neverCalledWith(_path().default.normalize(`${fixtures}/amd/b.js`)));
    });
    it('returns the precomputed list of a cached entry point', function () {
      var filename = `${fixtures}/amd/a.js`;
      var directory = `${fixtures}/amd`;
      var cache = {
        // Shouldn't process the first file's tree
        [filename]: {
          pathMap: {
            dependencies: []
          }
        }
      };
      var tree = dependencyTree({
        filename,
        directory,
        visited: cache
      });
      (0, _assert().default)(!tree.length);
    });
  });
  describe('module formats', function () {
    describe('amd', function () {
      testTreesForFormat('amd');
    });
    describe('commonjs', function () {
      testTreesForFormat('commonjs');
    });
    describe('es6', function () {
      beforeEach(function () {
        _this._directory = _path().default.normalize(`${fixtures}/es6`);
        mockes6();
      });
      testTreesForFormat('es6');
      it('resolves files that have jsx', function () {
        var filename = _path().default.normalize(`${_this._directory}/jsx.js`);

        var tree = dependencyTree({
          filename,
          directory: _this._directory
        });

        _assert().default.ok(tree[filename].includes(_path().default.normalize(`${_this._directory}/c.js`)));
      });
      it('resolves files with a jsx extension', function () {
        var filename = _path().default.normalize(`${_this._directory}/foo.jsx`);

        var tree = dependencyTree({
          filename,
          directory: _this._directory
        });

        _assert().default.ok(tree[filename].includes(_path().default.normalize(`${_this._directory}/b.js`)));
      });
      it('resolves files that have es7', function () {
        var filename = _path().default.normalize(`${_this._directory}/es7.js`);

        var tree = dependencyTree({
          filename,
          directory: _this._directory
        });

        _assert().default.ok(tree[filename].includes(_path().default.normalize(`${_this._directory}/c.js`)));
      });
    });
    describe('sass', function () {
      beforeEach(function () {
        mockSass();
      });
      testTreesForFormat('sass', '.scss');
    });
    describe('stylus', function () {
      beforeEach(function () {
        mockStylus();
      });
      testTreesForFormat('stylus', '.styl');
    });
    describe('less', function () {
      beforeEach(function () {
        mockLess();
      });
      testTreesForFormat('less', '.less');
    });
    describe('typescript', function () {
      beforeEach(function () {
        mockTS();
      });
      testTreesForFormat('ts', '.ts');
    });
  }); // skipping the webpack unit tests for now as it's not easy to wire up all the files together.
  // originally, in dependency-tree, the webpack.config.js is in the same directory of the index.js.
  // doing the same here will be confusing. instead, we have already e2e-tests in bit-bin of custom
  // module resolution, which takes advantage of the webpack config.

  describe.skip('webpack', function () {
    beforeEach(function () {
      // Note: not mocking because webpack's resolver needs a real project with dependencies;
      // otherwise, we'd have to mock a ton of files.
      _this._root = _path().default.join(__dirname, '../');
      _this._webpackConfig = `${_this._root}/webpack.config.js`;

      _this._testResolution = function (name) {
        var results = dependencyTree.toList({
          filename: `${fixtures}/webpack/${name}.js`,
          directory: _this._root,
          webpackConfig: _this._webpackConfig,
          filter: function filter(filename) {
            return filename.indexOf('filing-cabinet') !== -1;
          }
        });

        _assert().default.ok(results.some(function (filename) {
          return filename.indexOf('node_modules/filing-cabinet') !== -1;
        }));
      };
    });
    it('resolves aliased modules', function () {
      _this.timeout(5000);

      _this._testResolution('aliased');
    });
    it('resolves unaliased modules', function () {
      _this.timeout(5000);

      _this._testResolution('unaliased');
    });
  });
  describe('requirejs', function () {
    beforeEach(function () {
      (0, _mockFs().default)({
        root: {
          'lodizzle.js': 'define({})',
          'require.config.js': `
            requirejs.config({
              baseUrl: './',
              paths: {
                F: './lodizzle.js'
              }
            });
          `,
          'a.js': `
            define([
              'F'
            ], function(F) {

            });
          `,
          'b.js': `
            define([
              './lodizzle'
            ], function(F) {

            });
          `
        }
      });
    });
    it('resolves aliased modules', function () {
      var tree = dependencyTree({
        filename: 'root/a.js',
        directory: 'root',
        config: 'root/require.config.js'
      });

      var filename = _path().default.resolve(process.cwd(), 'root/a.js');

      var aliasedFile = _path().default.resolve(process.cwd(), 'root/lodizzle.js');

      (0, _assert().default)(tree[filename].includes(_path().default.normalize('root/lodizzle.js')));
    });
    it('resolves non-aliased paths', function () {
      var tree = dependencyTree({
        filename: 'root/b.js',
        directory: 'root',
        config: 'root/require.config.js'
      });

      var filename = _path().default.resolve(process.cwd(), 'root/b.js');

      var aliasedFile = _path().default.resolve(process.cwd(), 'root/lodizzle.js');

      _assert().default.ok(tree[filename].includes(_path().default.normalize('root/lodizzle.js')));
    });
  });
  describe('when a filter function is supplied', function () {
    it('uses the filter to determine if a file should be included in the results', function () {
      var directory = _path().default.normalize(`${fixtures}/onlyRealDeps`);

      var filename = _path().default.normalize(`${directory}/a.js`);

      var tree = dependencyTree({
        filename,
        directory,
        // Skip all 3rd party deps
        filter: function filter(filePath, moduleFile) {
          _assert().default.ok(require.resolve('debug'));

          _assert().default.ok(moduleFile.includes(_path().default.normalize('onlyRealDeps/a.js')));

          return filePath.indexOf('node_modules') === -1;
        }
      });
      var subTree = tree[filename];

      _assert().default.ok(Object.keys(tree).length);

      var has3rdPartyDep = Object.keys(subTree).some(function (dep) {
        return dep === require.resolve('debug');
      });

      _assert().default.ok(!has3rdPartyDep);
    });
  });
  describe('when given a CJS file with lazy requires', function () {
    beforeEach(function () {
      (0, _mockFs().default)({
        [`${__dirname}/cjs`]: {
          'foo.js': 'module.exports = function(bar = require("./bar")) {};',
          'bar.js': 'module.exports = 1;'
        }
      });
    });
    it('includes the lazy dependency', function () {
      var directory = `${__dirname}/cjs`;

      var filename = _path().default.normalize(`${directory}/foo.js`);

      var tree = dependencyTree({
        filename,
        directory
      });

      _assert().default.ok(tree[filename].includes(_path().default.normalize(`${directory}/bar.js`)));
    });
  });
  describe('when given an es6 file using CJS lazy requires', function () {
    beforeEach(function () {
      (0, _mockFs().default)({
        [`${__dirname}/es6`]: {
          'foo.js': 'export default function(bar = require("./bar")) {};',
          'bar.js': 'export default 1;'
        }
      });
    });
    describe('and mixedImport mode is turned on', function () {
      it('includes the lazy dependency', function () {
        var directory = `${__dirname}/es6`;

        var filename = _path().default.normalize(`${directory}/foo.js`);

        var tree = dependencyTree({
          filename,
          directory,
          detective: {
            es6: {
              mixedImports: true
            }
          }
        });

        _assert().default.ok(tree[filename].includes(_path().default.normalize(`${directory}/bar.js`)));
      });
    });
  });
  describe('when given an es6 file using dynamic imports', function () {
    beforeEach(function () {
      (0, _mockFs().default)({
        [`${__dirname}/es6`]: {
          'foo.js': 'import("./bar");',
          'bar.js': 'export default 1;'
        }
      });
    });
    it('includes the dynamic import', function () {
      var directory = _path().default.normalize(`${__dirname}/es6`);

      var filename = _path().default.normalize(`${directory}/foo.js`);

      var tree = dependencyTree({
        filename,
        directory
      });
      var subTree = tree[filename];

      _assert().default.ok(!(`${directory}/bar.js` in subTree));
    });
  });
  describe('when a dependency of the main file is not supported', function () {
    beforeEach(function () {
      (0, _mockFs().default)({
        [`${__dirname}/baz`]: {
          'foo.js': 'require("./bar.json");',
          'bar.json': '{ "main": "I\'m a simple JSON object" }'
        }
      });
    });
    it('should include it as a dependency and not throw an error', function () {
      var directory = _path().default.normalize(`${__dirname}/baz`);

      var filename = _path().default.normalize(`${directory}/foo.js`);

      var tree = dependencyTree({
        filename,
        directory
      });

      _assert().default.ok(`${directory}/bar.json` in tree);
    });
  }); // nodeModulesConfig is a feature added to dependency-tree and filing-cabinet to support
  // "module" attribute of package.json, see here what this attribute is good for:
  // https://github.com/rollup/rollup/wiki/pkg.module
  // the commit of supporting it in filing-cabinet is here: https://github.com/dependents/node-filing-cabinet/commit/abef861a5a725b29c2342d01de94c6e2dd881aa0

  describe.skip('when given a CJS file with module property in package.json', function () {
    beforeEach(function () {
      (0, _mockFs().default)({
        [`${__dirname}/es6`]: {
          'module.entry.js': 'import * as module from "module.entry"',
          node_modules: {
            'module.entry': {
              'index.main.js': 'module.exports = () => {};',
              'index.module.js': 'module.exports = () => {};',
              'package.json': '{ "main": "index.main.js", "module": "index.module.js" }'
            }
          }
        }
      });
    });
    it('it includes the module entry as dependency', function () {
      var directory = `${__dirname}/es6`;
      var filename = `${directory}/module.entry.js`;
      var tree = dependencyTree({
        filename,
        directory,
        nodeModulesConfig: {
          entry: 'module'
        }
      });
      var subTree = tree[filename];

      _assert().default.ok(`${directory}/node_modules/module.entry/index.module.js` in subTree);
    });
  });
  describe('Config', function () {
    describe('when cloning', function () {
      describe('and a detective config was set', function () {
        it('retains the detective config in the clone', function () {
          var detectiveConfig = {
            es6: {
              mixedImports: true
            }
          };
          var config = new (_Config().default)({
            detectiveConfig,
            filename: 'foo',
            directory: 'bar'
          });
          var clone = config.clone();

          _assert().default.deepEqual(clone.detectiveConfig, detectiveConfig);
        });
      });
    });
  });
  describe('when a dependency has missing packages and is retrieved from the cache (visited)', function () {
    beforeEach(function () {
      (0, _mockFs().default)({
        [`${__dirname}/baz`]: {
          'foo.js': 'require("non-exist-foo-pkg");',
          'bar.js': 'require("./foo"); require("non-exist-bar-pkg")',
          'baz.js': 'require("./foo"); require("./bar"); require("non-exist-baz-pkg")'
        }
      });
    });
    it('should not override the cache with wrong packages', function () {
      var directory = _path().default.normalize(`${__dirname}/baz`);

      var fooFile = _path().default.normalize(`${directory}/foo.js`);

      var barFile = _path().default.normalize(`${directory}/bar.js`);

      var bazFile = _path().default.normalize(`${directory}/baz.js`);

      var nonExistent = {};
      var config = {
        directory,
        nonExistent,
        visited: {}
      };
      config.filename = fooFile;
      dependencyTree(config);
      (0, _chai().expect)(nonExistent[fooFile]).to.deep.equal(['non-exist-foo-pkg']);
      config.filename = barFile;
      dependencyTree(config);
      (0, _chai().expect)(nonExistent[fooFile]).to.deep.equal(['non-exist-foo-pkg']);
      (0, _chai().expect)(nonExistent[barFile]).to.deep.equal(['non-exist-bar-pkg']);
      config.filename = bazFile;
      dependencyTree(config);
      (0, _chai().expect)(nonExistent[fooFile]).to.deep.equal(['non-exist-foo-pkg']);
      (0, _chai().expect)(nonExistent[barFile]).to.deep.equal(['non-exist-bar-pkg']);
      (0, _chai().expect)(nonExistent[bazFile]).to.deep.equal(['non-exist-baz-pkg']);
    });
  });
  describe('passing css files and then javascript files', function () {
    beforeEach(function () {
      (0, _mockFs().default)({
        [`${__dirname}/baz`]: {
          'base.scss': 'li {} a {}',
          // don't change the content. it crash only with this for some reason
          'index.jsx': "require('some-module');"
        }
      });
    });
    it('should not crash with "RangeError: Maximum call stack size exceeded" error', function () {
      var directory = _path().default.normalize(`${__dirname}/baz`);

      var baseFile = _path().default.normalize(`${directory}/base.scss`);

      var indexFile = _path().default.normalize(`${directory}/index.jsx`);

      var config = {
        directory
      };
      config.filename = baseFile;
      dependencyTree(config);
      config.filename = indexFile;
      var dependencies = dependencyTree(config);
      (0, _chai().expect)(dependencies).to.be.ok;
    });
  });
});
//# sourceMappingURL=index.spec.js.map