"use strict";

var stylable = require('stylable');

module.exports = function (src) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var css = stylable.safeParse(src);
  var dependencies = {};

  var addDependency = function addDependency(dependency) {
    if (!dependencies[dependency]) {
      dependencies[dependency] = {};
    }
  };

  var addImportSpecifier = function addImportSpecifier(dependency, importSpecifier) {
    if (dependencies[dependency].importSpecifiers) {
      dependencies[dependency].importSpecifiers.push(importSpecifier);
    } else {
      dependencies[dependency].importSpecifiers = [importSpecifier];
    }
  };

  css.walkRules(function (rule) {
    var stFrom = rule.nodes.find(function (node) {
      return node.prop === '-st-from';
    });
    if (!stFrom) return;
    var stFromValue = stFrom.value.replace(/["']/g, '');
    addDependency(stFromValue);
    var stNamed = rule.nodes.find(function (node) {
      return node.prop === '-st-named';
    });
    if (!stNamed) return;
    var specifierValue = {
      isDefault: false,
      // @todo,
      name: stNamed.value
    };
    addImportSpecifier(stFromValue, specifierValue);
  });
  return dependencies;
};
//# sourceMappingURL=index.js.map