"use strict";

function _chai() {
  var data = require("chai");

  _chai = function _chai() {
    return data;
  };

  return data;
}

function _assert() {
  var data = _interopRequireDefault(require("assert"));

  _assert = function _assert() {
    return data;
  };

  return data;
}

function _() {
  var data = _interopRequireDefault(require("./"));

  _ = function _() {
    return data;
  };

  return data;
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-env mocha */
describe('detective-typescript', function () {
  var ast = {
    type: 'Program',
    body: [{
      type: 'VariableDeclaration',
      declarations: [{
        type: 'VariableDeclarator',
        id: {
          type: 'Identifier',
          name: 'x'
        },
        init: {
          type: 'Literal',
          value: 4,
          raw: '4'
        }
      }],
      kind: 'let'
    }]
  };
  it('accepts an ast', function () {
    var deps = (0, _().default)(ast);
    var depsKeys = Object.keys(deps);
    (0, _assert().default)(!depsKeys.length);
  });
  it('retrieves the dependencies of modules', function () {
    var deps = (0, _().default)('import {foo, bar} from "mylib";');
    var depsKeys = Object.keys(deps);
    (0, _assert().default)(depsKeys.length === 1);
    (0, _chai().expect)(deps).to.have.property('mylib');
  });
  it('retrieves the re-export dependencies of modules', function () {
    var deps = (0, _().default)('export {foo, bar} from "mylib";');
    var depsKeys = Object.keys(deps);
    (0, _assert().default)(depsKeys.length === 1);
    (0, _chai().expect)(deps).to.have.property('mylib');
  });
  it('retrieves the re-export * dependencies of modules', function () {
    var deps = (0, _().default)('export * from "mylib";');
    var depsKeys = Object.keys(deps);
    (0, _assert().default)(depsKeys.length === 1);
    (0, _chai().expect)(deps).to.have.property('mylib');
  });
  it('handles multiple imports', function () {
    var deps = (0, _().default)('import {foo, bar} from "mylib";\nimport "mylib2"');
    var depsKeys = Object.keys(deps);
    (0, _assert().default)(depsKeys.length === 2);
    (0, _chai().expect)(deps).to.have.property('mylib');
    (0, _chai().expect)(deps).to.have.property('mylib2');
  });
  it('handles mixed imports of typescript and javascript', function () {
    var deps = (0, _().default)('import {foo, bar} from "mylib";\nconst mylib2 = require("mylib2");');
    var depsKeys = Object.keys(deps);
    (0, _assert().default)(depsKeys.length === 2);
    (0, _chai().expect)(deps).to.have.property('mylib');
    (0, _chai().expect)(deps).to.have.property('mylib2');
  });
  it('handles default imports', function () {
    var deps = (0, _().default)('import foo from "foo";');
    var depsKeys = Object.keys(deps);
    (0, _assert().default)(depsKeys.length === 1);
    (0, _chai().expect)(deps).to.have.property('foo');
  });
  it('retrieves dependencies from modules using "export ="', function () {
    var deps = (0, _().default)('import foo = require("mylib");');
    var depsKeys = Object.keys(deps);
    (0, _assert().default)(depsKeys.length === 1);
    (0, _chai().expect)(deps).to.have.property('mylib');
  });
  it('retrieves dependencies when using javascript syntax', function () {
    var deps = (0, _().default)('var foo = require("foo");');
    var depsKeys = Object.keys(deps);
    (0, _assert().default)(depsKeys.length === 1);
    (0, _chai().expect)(deps).to.have.property('foo');
  });
  it('returns an empty list for empty files', function () {
    var deps = (0, _().default)('');
    var depsKeys = Object.keys(deps);

    _assert().default.equal(depsKeys.length, 0);
  });
  it('throws when content is not provided', function () {
    _assert().default.throws(function () {
      (0, _().default)();
    }, Error, 'src not given');
  });
  it('does not throw with jsx in a module', function () {
    _assert().default.doesNotThrow(function () {
      (0, _().default)("import foo from 'foo'; var baz = <baz>bar;");
    });
  });
});
//# sourceMappingURL=index.spec.js.map